/*	awm - A Window Manager: Minimalist Window Manager for X		*/
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/wait.h>
#include <unistd.h>
#include <signal.h>

#include <X11/cursorfont.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>

/*  Keymappings  */
#define KEY_ONTOP       XK_a
	/* XK_b */
#define KEY_CLOSE	XK_c
	/* XK_d */
	/* XK_e */
	/* XK_f */
	/* XK_g */
#define KEY_SPLIT	XK_h
#define KEY_ICONIC      XK_i
	/* XK_j */
	/* XK_k */
	/* XK_l */
	/* XK_m */
	/* XK_n */
	/* XK_o */
	/* XK_p */
	/* XK_q */
#define KEY_RESTORE     XK_r
#define KEY_STICKY	XK_s
#define KEY_TILE	XK_t
#define KEY_UNMAP	XK_u
#define KEY_VERTICAL	XK_v
	/* XK_w */
#define KEY_MAX		XK_x
	/* XK_y */
#define KEY_ZOOM	XK_z

#define KEY_PREVDESK	XK_Left
#define KEY_NEXTDESK	XK_Right
#define KEY_UP		XK_Up
#define KEY_DOWN	XK_Down

#define KEY_KILL	XK_Escape
#define KEY_NEXT        XK_space
#define KEY_NEW         XK_Return

#define FALSE		0
#define TRUE		1

/* Border colors indicate which window has focus. Default selected to match desktop bg so I only see a border around the active window */
#define BACKGROUND	"gray"
#define ACTIVE_WIN	"blue"

/* Subtle border colors may require a larger default border */
#define DEFAULT_BORDER	1
#define BW		2*DEFAULT_BORDER

/* Default xterm program launched with KEY_NEW */
#define TERMINAL	"xterm"

/* Size of divider between windows when tiled or otherwise awm partitioned */
#define TILE_BORDER	6

/* Should be indentical to TILE_BORDER or 0 for best layout aesthetics */
#define WINBORDER	0	

/* Useful macros for mouse operations, etc  */
#define ChildMask	(SubstructureRedirectMask|SubstructureNotifyMask)
#define ButtonMask	(ButtonPressMask|ButtonReleaseMask)
#define MouseMask	(ButtonMask|PointerMotionMask)

#define setmouse(w, x, y) \
	XWarpPointer(dpy, None, w, 0, 0, 0, 0, x, y)

#define grab() \
	(XGrabPointer(dpy, root, False, MouseMask, GrabModeAsync, GrabModeAsync, None, move_curs, CurrentTime) == GrabSuccess)


typedef struct Client Client;

struct Client {
	Client *next;
	Window window;
	int x;
	int y;
	int width;
	int height;
	int sticky;
	int iconic;
};

Atom _XA_WM_PROTOCOLS;
Atom _XA_WM_DELETE_WINDOW;

/* Function prototypes */
Client *find_client(Window w);
Client *make_new_client(Window w);
Client *nextwindow(Client *c);

void remove_client(Client *c);

void handle_key_event(XKeyEvent *e);
void handle_button_event(XButtonEvent *e);
void handle_configure_request(XConfigureRequestEvent *e);
void handle_enter_event(XCrossingEvent *e);
void handle_map_request(XMapRequestEvent *e);
void handle_unmap_event(XUnmapEvent *e);

void handle_destroy_event(XDestroyWindowEvent *e);

int handle_xerror(Display *dpy, XErrorEvent *e);
int ignore_xerror(Display *dpy, XErrorEvent *e);
void handle_signal(int signo);

void drag(Client *c);
void move(Client *c);
void recalc(Client *c, int x1, int y1, int x2, int y2);

void resize(Client *c);
void sweep(Client *c);
void zoom(Client *c);
void full(Client *c);

void goto_next(Client *c);
void switch_vdesk(int x, int y);

void tile(int n);
void tile_vertical(int n);
void tile_horizontal(int n);

void set_on_top(Client *c);
void smart_raise(Client *c);

int test(Client *c);
int count_windows();
int count_tiled_windows();

Display		*dpy;
int		screen;
Window		root;
GC		invert_gc;

XFontStruct	*font;

Client		*head_client = NULL;
Client		*current = NULL;
Client		*on_top = NULL;

Cursor		move_curs;

char            *opt_bg = BACKGROUND;
char		*opt_fg = ACTIVE_WIN;

XColor          bg, fg;

int		xdesk = 0;
int		ydesk = 0;
int		xmax, ymax;
int		remember = 0;

void setup_display() {
	XGCValues gv;
	XSetWindowAttributes attr;
	XColor dummy;
	int *i;
	int keys_to_grab[] = {
		KEY_NEW, KEY_KILL, KEY_NEXT, KEY_MAX, KEY_PREVDESK, KEY_NEXTDESK, KEY_UP, KEY_DOWN,
		KEY_TILE, KEY_SPLIT, KEY_VERTICAL, KEY_STICKY, KEY_ZOOM, KEY_ICONIC, KEY_RESTORE,
		KEY_ONTOP, KEY_UNMAP, KEY_CLOSE,
		XK_0, XK_1, XK_2, XK_3, XK_4, XK_5, XK_6, XK_7, XK_8, XK_9, 0
	};

	dpy = XOpenDisplay("");

	if (!dpy) { 
		exit(1);
	}

	XSetErrorHandler(handle_xerror);

	screen = DefaultScreen(dpy);
	root = RootWindow(dpy, screen);

	XAllocNamedColor(dpy, DefaultColormap(dpy, screen), opt_bg, &bg, &dummy);
	XAllocNamedColor(dpy, DefaultColormap(dpy, screen), opt_fg, &fg, &dummy);
	_XA_WM_PROTOCOLS = XInternAtom (dpy, "WM_PROTOCOLS", False);
	_XA_WM_DELETE_WINDOW = XInternAtom (dpy, "WM_DELETE_WINDOW", False);

	font = XLoadQueryFont(dpy, "*");
	move_curs = XCreateFontCursor(dpy, XC_fleur);

	gv.function = GXinvert;
	gv.subwindow_mode = IncludeInferiors;
	gv.line_width = 1;
	gv.font = font->fid;
	invert_gc = XCreateGC(dpy, root, GCFunction | GCSubwindowMode | GCLineWidth | GCFont, &gv);

	attr.event_mask = ChildMask | PropertyChangeMask | ButtonMask;

	XChangeWindowAttributes(dpy, root, CWEventMask, &attr);

	for (i = keys_to_grab; *i; i++) {
		XGrabKey(dpy, XKeysymToKeycode(dpy, *i), Mod4Mask | Mod2Mask , root, True, GrabModeAsync, GrabModeAsync);
		XGrabKey(dpy, XKeysymToKeycode(dpy, *i), Mod4Mask , root, True, GrabModeAsync, GrabModeAsync);
	}
}

/* Take control of the existing windows.  We can't have an on-top window yet because we haven't been running */
void scan_windows() {
	unsigned int i, nwins;
	Window dw1, dw2, *wins;
	XWindowAttributes attr;

	XQueryTree(dpy, root, &dw1, &dw2, &wins, &nwins);

	for (i = nwins; i > 0; i--) {
		XGetWindowAttributes(dpy, wins[i-1], &attr);

		if (!attr.override_redirect && attr.map_state == IsViewable)
			make_new_client(wins[i-1]);
	}
	XFree(wins);
}


void spawn(char *cmd) {
	pid_t pid = fork();

	if (pid == 0) {
        	execlp(cmd, cmd, NULL);
	}
}


void handle_signal(int signo) {

	if (signo == SIGCHLD) {
		wait(NULL);
		return;
	}

	while(head_client)
		remove_client(head_client);

	XSetInputFocus(dpy, PointerRoot, RevertToPointerRoot, CurrentTime);
	XCloseDisplay(dpy);
	exit(0);

}

int handle_xerror(Display *dpy, XErrorEvent *e) {
//	Client *c = find_client(e->resourceid);
//	remove_client(c);
	return 0;
}

int ignore_xerror(Display *dpy, XErrorEvent *e) {
	return 0;
}


Client *find_client(Window w) {
	Client *c;

	for (c = head_client; c; c = c->next)
		if (w == c->window) return c;

	return NULL;
}


void send_config(Client *c) {
	XConfigureEvent ce;

	ce.type = ConfigureNotify;
	ce.event = c->window;
	ce.window = c->window;
	ce.x = c->x;
	ce.y = c->y;
	ce.width = c->width;
	ce.height = c->height;
	ce.border_width = 0;
	ce.above = None;
	ce.override_redirect = 0;

	XSendEvent(dpy, c->window, False, StructureNotifyMask, (XEvent *)&ce);
}


void set_mouse(Client *c) {
	setmouse(c->window, (c->width >> 1), (c->height >> 1));
}

/* Possible to callefor a window that ISN'T where we have the pointer: don't muck with "current pointer" unless explicitly necessary  */
void remove_client(Client *c) {
	Client *p;

	if (c) {

/* Notice when we delete the on-top window */
	        if (on_top == c)
        	        on_top = NULL;

		if (current == c)
			current = NULL;

		if (head_client == c)
			head_client = c->next;
		else
			for (p = head_client; p && p->next; p = p->next)
				if (p->next == c)
					p->next = c->next;
	
		free(c);
	}
}


void unmap_all_windows() {
	Client *c;

	for (c = head_client; c; c = c->next) {
		XUnmapWindow(dpy,c->window);
		c->iconic = TRUE;
	}
}

void restore_all_windows() {
	Client *c;

	for (c = head_client; c; c = c->next) {
		if (c->iconic) {
			XMapWindow(dpy, c->window);
			c->iconic = FALSE;
		}
	}
}


void handle_key_event(XKeyEvent *e) {
	int count;
	XEvent ev;

	Client *c = find_client(e->window);
	KeySym key = XKeycodeToKeysym(dpy,e->keycode,0);

	if (!c)
		c = current;

	if (c) {
		switch (key) {

			case KEY_STICKY:
				c->sticky ^= TRUE;
				return;

			case KEY_KILL:
				XKillClient(dpy, c->window);
				return;

			case KEY_CLOSE: /* A kinder, gentler kill */
				ev.xclient.type = ClientMessage;
				ev.xclient.display = dpy;
				ev.xclient.window = c->window;
				ev.xclient.message_type = _XA_WM_PROTOCOLS;
				ev.xclient.format = 32;
				ev.xclient.data.l[0] = _XA_WM_DELETE_WINDOW;
				ev.xclient.data.l[1] = CurrentTime;
				ev.xclient.data.l[2] = 0;
				ev.xclient.data.l[3] = 0;
				XSendEvent(dpy, c->window, False, NoEventMask, &ev);
				XSync(dpy, False);
				return;

			case KEY_ONTOP:
				set_on_top(c);
				smart_raise(c);
				return;
		
		}
	}

/* Don't do any of these if none of the visible windows is the active one */
	if (test(c)) {
		switch (key) {

			case KEY_MAX:
				full(c);
				return;

			case KEY_ZOOM:
				zoom(c);
				return;

			case KEY_ICONIC:
				XUnmapWindow(dpy, c->window);
				return;
		}
	}

		switch (key) {

			case KEY_NEXT:
				goto_next(c);
				return;

			case XK_0: case XK_1: case XK_2: case XK_3: case XK_4:
			case XK_5: case XK_6: case XK_7: case XK_8: case XK_9:
				switch_vdesk(key - XK_0 - xdesk, -ydesk);
				return;
		
			case KEY_PREVDESK:
				if (ydesk == 0)
					switch_vdesk(-1, 0);
                       		return;

			case KEY_NEXTDESK:
				if (ydesk == 0)
					switch_vdesk(1, 0);
				return;


			case KEY_UP: case KEY_DOWN:
				switch (ydesk) {
					case 0:
						remember = xdesk;
						switch_vdesk(-xdesk, 1);
						return;
					case 1:
						switch_vdesk(remember, -1);
						return;
				}

			case KEY_NEW:
				spawn(TERMINAL);
				return;

			case KEY_UNMAP:
				unmap_all_windows();
				return;

			case KEY_RESTORE:
				restore_all_windows();
				return;

	}

/* These are conditional on a window existing in the active desktop, as all tile routines assume a cyclic window list */
	count = count_tiled_windows();

	if (count) {
		switch(key) {
			case KEY_VERTICAL:
				tile_vertical(count);
				return;

			case KEY_SPLIT:
				tile_horizontal(count);
				return;

			case KEY_TILE:
				tile(count);
				return;
		}
	}	
}


void handle_button_event(XButtonEvent *e) {
	Client *c = find_client(e->window);

	if (c && e->window != root) {
		switch (e->button) {
			case Button1:
				move(c);
				break;

			case Button2:
				sweep(c);
				resize(c);
				break;
	
			case Button3:
				smart_raise(c);
				break;
		}
	}
	switch (e->button) {
		case Button4:
			if (ydesk == 0)
				switch_vdesk(-1, 0);
			break;
		case Button5:
			if (ydesk == 0)
				switch_vdesk(1, 0);
			break;
	}
}


void zoom(Client *c) {
	c->width  = 3*xmax/4;
	c->height = 3*ymax/4;
	c->x = xmax/8;
	c->y = ymax/8;
	resize(c);
}

void full(Client *c) {
	c->x = c->y = WINBORDER;
	c->width = xmax - BW - 2*WINBORDER;
	c->height = ymax - BW - 2*WINBORDER;
	resize(c);
}


void handle_configure_request(XConfigureRequestEvent *e) {
	Client *c = find_client(e->window);
	XWindowChanges wc;
	
	wc.sibling = e->above;
	wc.stack_mode = e->detail;

	if (c) {
		if (e->value_mask & CWX) c->x = e->x;
		if (e->value_mask & CWY) c->y = e->y;
		if (e->value_mask & CWWidth) c->width = e->width;
		if (e->value_mask & CWHeight) c->height = e->height;

		wc.border_width = 0;
	}
		wc.x = e->x;
		wc.y = e->y;
		wc.width = e->width;
		wc.height = e->height;
		XConfigureWindow(dpy, e->window, e->value_mask, &wc);

	if (c)
		send_config(c);
}


void init_position(Client *c) {

	if ((c->x == 0) && (c->y == 0)) {

		if ((xmax < c->width) || (ymax < c->height)) {
			full(c);
		}
		else {
	       		c->x = (xmax - c->width)/2;
			c->y = (ymax - c->height)/2;
			resize(c);
		}
	}
}

void handle_map_request(XMapRequestEvent *e) {

	Client *c = find_client(e->window);

	if (c) {
		c->iconic = FALSE;
		smart_raise(c);
	}
	else { 
		c = make_new_client(e->window);
		init_position(c);
		smart_raise(c);
	}
}

void handle_unmap_event(XUnmapEvent *e) {
	Client *c = find_client(e->window);

	if (c)
		c->iconic = TRUE;
}

void handle_destroy_event(XDestroyWindowEvent *e) {
	Client *c = find_client(e->window);

	remove_client(c);
}

/* This is where we handle window border highlighting. */
void handle_enter_event(XCrossingEvent *e) {
	Client *c = find_client(e->window);
	if (c) {
		if (current && c!=current) {
			XSetWindowBorder(dpy, current->window, bg.pixel);
		}

		XSetWindowBorder(dpy, c->window, fg.pixel );
		current = c;
		XSetInputFocus(dpy, c->window, RevertToPointerRoot, CurrentTime);

		XGrabButton(dpy, AnyButton, Mod4Mask, c->window, False,
			ButtonMask, GrabModeAsync, GrabModeSync, None, None);

		XGrabButton(dpy, AnyButton, Mod4Mask | Mod2Mask, c->window, False,
			ButtonMask, GrabModeAsync, GrabModeSync, None, None);
	}
}


Client *make_new_client(Window w) {
	Client *c;
	XWindowAttributes attr;

#if 1 
	XTextProperty namen;
	namen.value = "";
#endif
	XGrabServer(dpy);
	c = (Client *)malloc(sizeof(Client));

	c->next = head_client;
	head_client = c;

	c->window = w;
	XGetWindowAttributes(dpy, c->window, &attr);

	c->x = attr.x;
	c->y = attr.y;
	c->width = attr.width;
	c->height = attr.height;
	c->sticky = FALSE;
	c->iconic = FALSE;

/* We don't use window names.  So, if you tell me that your name is one of the special classes that awm
   recognizes, you can get sticky windows from .Xclients */

#if 1
	if ( XGetWMName(dpy, c->window, &namen) ) {

		if (strcmp(namen.value, "STICKY") == 0)
			c->sticky = TRUE;
	}
#endif
	XSelectInput(dpy, c->window, EnterWindowMask | PropertyChangeMask);
        XAddToSaveSet(dpy, c->window);

	XSetWindowBorderWidth(dpy, c->window, DEFAULT_BORDER);
	XSetWindowBorder(dpy,c->window,bg.pixel);

	XSync(dpy, False);
	XUngrabServer(dpy);

	return c;
}


void outline(Client *c) {
	XDrawRectangle(dpy, root, invert_gc, c->x, c->y, c->width, c->height);
}


void recalc(Client *c, int x1, int y1, int x2, int y2) {

	c->width = abs(x1 - x2);
	c->height = abs(y1 - y2);
	c->x = (x1 <= x2) ? x1 : x1 - c->width;
	c->y = (y1 <= y2) ? y1 : y1 - c->height;
}

void sweep(Client *c) {
	XEvent ev;
	int old_cx = c->x;
	int old_cy = c->y;

	if (!grab())
		return;
	
	XGrabServer(dpy);
	outline(c);
	setmouse(c->window, c->width - 1, c->height - 1);

	for (;;) {
		XMaskEvent(dpy, MouseMask, &ev);
		switch (ev.type) {

			case MotionNotify:
				outline(c); /* clear */
				recalc(c, old_cx, old_cy, ev.xmotion.x, ev.xmotion.y);
				outline(c);
				break;

			case ButtonRelease:
				outline(c); /* clear */
				XUngrabServer(dpy);
				XUngrabPointer(dpy, CurrentTime);
				return;
		}
	}
}

void drag(Client *c) {
	XEvent ev;
	int x1, y1;
	int old_cx = c->x;
	int old_cy = c->y;

	Window dw1, dw2;
	int t1, t2;
	unsigned int t3;

	if (!grab())
		return;

	XQueryPointer(dpy, root, &dw1, &dw2, &x1, &y1, &t1, &t2, &t3);
	XGrabServer(dpy);

	outline(c);

	for (;;) {
		XMaskEvent(dpy, MouseMask, &ev);
		switch (ev.type) {

			case MotionNotify:
				outline(c); /* clear */
				c->x = old_cx + (ev.xmotion.x - x1);
				c->y = old_cy + (ev.xmotion.y - y1);
				outline(c);
				break;

			case ButtonRelease:
				outline(c); /* clear */
				XUngrabServer(dpy);
				XUngrabPointer(dpy, CurrentTime);
				return;
		}
	}
}

void move(Client *c) {

	if (c) {
		drag(c);
		XMoveWindow(dpy, c->window, c->x, c->y);
		send_config(c);
	}
}

void resize(Client *c) {
	if (c) {
		XMoveResizeWindow(dpy, c->window, c->x, c->y, c->width, c->height);
		send_config(c);
	}
}

/* Raise the selected window.  But make sure that the "on-top" window is still the most visible window */

void smart_raise(Client *c) {

	XMapRaised(dpy, c->window);

	if (on_top)
		XMapRaised(dpy, on_top->window);
	
}

/* Decide if the window belongs to the current virtual desktop. We define that as meaning that the upper left corner is visible */
int test(Client *c) {

    if (c) {

	if ((c->x < 0 ) || (c->y < 0) || (c->x >= xmax) || (c->y >= ymax))
		return 0;
	
	return 1;

	}

    else
	return 0;
}



int count_windows() {

	int total = 0;
	Client *c = head_client;

	while (c)
	{
		if (!c->iconic)
			total += test(c);
		c = c->next;
	}

	return total;
}

int count_tiled_windows() {

	int total = 0;
	Client *c = head_client;

	while (c)
	{
		if (!c->sticky && !c->iconic)
			total += test(c);
		c = c->next;
	}

	return total;

}


Client *nextwindow(Client *c) {

	do {
		if ((c) && (c->next))
		       	c = c->next;
		else
			c = head_client;
		
	} while ( (test(c) == 0) || (c->iconic) );

	return c;
}


void goto_next(Client *c) {
	Client *newc = c;

	if (!newc)
		newc = head_client;

/* Don't execute this if there are no windows, because nextwindow MUST return since it treats the window list as cyclic */
	if (count_windows() && newc) {
		newc = nextwindow(newc);

/* Minor corner case when only one window */
		if (newc != c) {
			smart_raise(newc);
			set_mouse(newc);
		}
	}
}

/* If you have more than nine windows this isn't appropriate, so we trap it off */
void tile(int number_windows) {

	Client *c;
	int i, j, maxh, maxv;

	int tile_rows = 2;
	int tile_cols = 2;

	if (number_windows < 10) {

		if (number_windows > 4)
			tile_cols++;

		if (number_windows > 6)
			tile_rows++;

		c = NULL;

		maxh = (xmax - TILE_BORDER * (tile_cols - 1) - 2*WINBORDER) / tile_cols;
		maxv = (ymax - TILE_BORDER * (tile_rows - 1) - 2*WINBORDER) / tile_rows;

		for (i = number_windows-1; i >= 0; i--) {
			c = nextwindow(c);
		
			c->width  = maxh - BW;
			c->height = maxv - BW;
		
			j = i % tile_cols;
			c->x = WINBORDER + j * (maxh + TILE_BORDER);

			j = i / tile_cols;
			c->y = WINBORDER + j * (maxv + TILE_BORDER);

			resize(c);
		}
	}

}

/* Make the currently selected window the "on-top" window.  Setting a window sticky is a separate operation (!) */
void set_on_top(Client *c) {

	if (on_top == c)
		on_top = NULL;
	else 
		on_top = c;
}




void tile_horizontal(int number_windows){

	int max, i;
	Client *c;
	int delta = WINBORDER;

	c = NULL;
	max = ymax - 2*WINBORDER;	

	if (number_windows > 1)
	  max = (max - (TILE_BORDER * (number_windows-1))) / number_windows;

	for (i=0; i < number_windows; i++) {
		c = nextwindow(c);
			
		c->x = WINBORDER;
		c->width = xmax - BW - 2*WINBORDER;
		c->y = delta;
		c->height = max - BW;

		delta += TILE_BORDER + max;
		resize(c);
	}
}

void tile_vertical(int number_windows) {

	int max, i;
	Client *c;
	int delta = WINBORDER;

	c = NULL;
	max = xmax - 2*WINBORDER;

	if (number_windows > 1)
	  max = (max - (TILE_BORDER * (number_windows-1))) / number_windows;

	for (i=0; i < number_windows; i++) {
		c = nextwindow(c);

		c->y = WINBORDER;
		c->height = ymax - BW - 2*WINBORDER;
		c->x = delta;
		c->width = max - BW;

		delta += TILE_BORDER + max;
		resize(c);
	}
}


void switch_vdesk(int delta_x, int delta_y) {
	Client *c;

	xdesk += delta_x;
	ydesk += delta_y;

	delta_x *= xmax;
	delta_y *= ymax;

	for (c = head_client; c; c = c->next) {

/* Be careful not to move sticky windows */
		if (!c->sticky) {

			c->x -= delta_x;
			c->y -= delta_y;

			XMoveWindow(dpy, c->window, c->x, c->y);
			send_config(c);
	   	}
	}


/* If there is only one visible window on this virtual desktop, make sure that it gets keyboard focus */
	if (count_tiled_windows() == 1)
		goto_next(NULL);

}


void event_loop() {

	XEvent ev;
	
	XNextEvent(dpy, &ev);
	switch (ev.type) {
		case KeyPress:
			handle_key_event(&ev.xkey);
			break;

		case ButtonPress:
			handle_button_event(&ev.xbutton);
			break;

		case ConfigureRequest:
			handle_configure_request(&ev.xconfigurerequest);
			break;

		case MapRequest:
			handle_map_request(&ev.xmaprequest);
			break;

		case EnterNotify:
			handle_enter_event(&ev.xcrossing);
			break;

		case UnmapNotify:
			handle_unmap_event(&ev.xunmap);
			break;

		case DestroyNotify:
			handle_destroy_event(&ev.xdestroywindow);
			break;
	}
}


int main(int argc, char *argv[]) {
	struct sigaction act;

	act.sa_handler = handle_signal;
	sigemptyset(&act.sa_mask);
	act.sa_flags = 0;
	sigaction(SIGTERM, &act, NULL);
	sigaction(SIGINT, &act, NULL);
	sigaction(SIGHUP, &act, NULL);
        sigaction(SIGCHLD, &act, NULL);

	setup_display();
	xmax = DisplayWidth(dpy, screen);
	ymax = DisplayHeight(dpy, screen);

	scan_windows();
	for (;;) { event_loop();}

	return 1;
}


